package terrain;

import java.io.*;

/*************************
 * TerrainGenerator
 * date: 2004-05-23
 * author: Kristian Lunde
 * This class generate a .ter file based on some input parameters.
 * The input parameters are:
 *		- Size/Points of the terrain, 
 *		- Stride size
 * 		- height factor, determine how steep the heightpoints get, higher values give steep heightpoints.
 *		- filepath and filename example: d:\terrain\terrain_5.ter
 ************************/

public class TerrainGenerator {

	private int size;
	private String filename;
	private PrintWriter output;
	
	private float stride;
	private float height_factor;
	private float[][][] matrix;

	/**
	 * main
	 * Kickstart the program.
	 */
	public static void main(String[] args){ 
		if(args.length != 4)
			System.out.print("Usage: TerrainGenerator terrain_size(Integer) stride_size(float) height_factor(float) filename/path");
		else
			new TerrainGenerator(Integer.parseInt(args[0]), Float.parseFloat(args[1]), Float.parseFloat(args[2]), args[3]);
	}

	/**
	 * Constructor
	 * @param int size - is the size of the heightfield to be created.
	 * @param float stride - is the length between the heightpoints.
	 * @param float factor - is a factor that determine if the heightpoints are large or small.
	 * @param String filename - is the filepath and filename of the file to be created.
	 */
	public TerrainGenerator(int size, float stride, float factor, String filename){
		this.size = size;
		this.stride = stride;
		this.height_factor = factor;
		this.filename = filename;
		
		matrix = new float[this.size][this.size][3];
		write();
	}
	
	/**
	 * write
	 * write the .ter file with the heighfield information.
	 */
	private void write(){
		try{
			output = new PrintWriter(new FileWriter(filename), true);
			
			float x = 0.0f;
			float y = 0.0f;
		
			output.print(size + "\n");
			for(int widthX = 0; widthX < size; widthX++){
				for(int widthY = 0; widthY < size; widthY++){
					output.print(widthX + "|" + widthY +"|" + y + "f|" + x + "f|" + (float)Math.random()* height_factor + "f\n");
					y += stride;
				}
				y = 0.0f;
				x += stride;
			}
			output.flush();
			output.close();
		
		}
		catch(IOException ie){ ie.printStackTrace(); }
	}

}