package terrain.filter;

/**************************
 * Based on a smoothing tutorial on lighthouse3d.com
 * Date: 2004-05-18
 * Author: Kristian Lunde
 * This class smooth a heightfield based on the bandfilter consept.
 *************************/

public class BandFilter extends Filter{

	/* Constructor */
	public BandFilter(){}
	
	/**
	 * Constructor
	 * @param float[][][] matrix - is the matrix that should be smoothed.
	 * @param float k - is the value that the smoothing algorithm should use to calculate the new values 
	 *									in the matrix.
	 */
	public BandFilter(float[][][] matrix, float k){
		super(matrix);
		setParameter(k);

		filter_name = "Band filter";
	}
	
	/**
	 * smooth
	 * this method smooth the matrix, this is done by calling recursive methods.
	 */
	public void smooth(){
		for(int n = 0; n < getMatrixSize(); n++){
			leftSmooth(0, n, getMatrixSize());
			rightSmooth(getMatrixSize() -1, n);
			downSmooth(n, 0, getMatrixSize() - 1);
			upSmooth(n, getMatrixSize() - 1);
		}
	}

	/**
	 * rightSmooth
	 * @param int x - is the size of the matrix x dimension.
	 * @param int y - is the size of the matrix y dimensjon.
	 * Smooth from left to right
	 */
	private void rightSmooth(int x, int y){
		if(x == 0) return;
		
		matrix[x][y][2] = (matrix[x - 1][y][2] *(1-getParameter())) + (matrix[x][y][2] * getParameter());
		rightSmooth(x - 1, y);
	}
	
	/**
	 * leftSmooth
	 * @param int x - is the size of the matrix x dimension.
	 * @param int y - is the size of the matrix y dimensjon.
	 * @param int arrayLength - is the size of the matrix left to smooth.
	 * Smooth from right to left
	 */
	private void leftSmooth(int x, int y, int arrayLength){
		if(x == (arrayLength-1)) return;
		
		matrix[x][y][2] = (matrix[x + 1][y][2] *(1 + getParameter())) + (matrix[x][y][2] * getParameter());
		leftSmooth(x + 1, y, arrayLength);
	}
	
	/**
	 * downSmooth
	 * @param int x - is the size of the matrix x dimension.
	 * @param int y - is the size of the matrix y dimensjon.
	 * @param int arrayLength - is the size of the matrix left to smooth.
	 * Smooth from the top to the bottom.
	 */
	private void downSmooth(int x, int y, int arrayLength){
		if(y == arrayLength) return;
		
		matrix[x][y][2] = (matrix[x][y + 1][2] * (1-getParameter())) + (matrix[x][y][2] * getParameter()); 
		downSmooth(x, y + 1, arrayLength);
	}
	
		/**
	 * rightSmooth
	 * @param int x - is the size of the matrix x dimension.
	 * @param int y - is the size of the matrix y dimensjon.
	 * Smooth from the bottom to the top.
	 */
	private void upSmooth(int x, int y){
		if(y == 0) return;

		matrix[x][y][2] = (matrix[x][y - 1][2] * (1-getParameter())) + (matrix[x][y][2] * getParameter());
		upSmooth(x, y - 1);
	}
}