package terrain.filter;

/******************************
 * Filter
 * date: 2004-05-21
 * author: Kristian Lunde
 * This class is a abstract superclass for all the filters in the terrain.filter package.
 *****************************/

public abstract class Filter {

	protected float[][][] matrix;
	protected int matrix_size;
	protected String filter_name;
	protected float input_parameter;

	/* Empty constructor */
	public Filter(){}
	
	/**
	 * Constructor
	 * @param float[][][] matrix - set the matrix that should be smoothed,
	 */
	public Filter(float[][][] matrix){ 
		this.matrix = matrix; 
		setMatrixSize(matrix.length);
	}
	
	/**
	 * setMatrix
	 * @param float[][][] m - set the matrix that should be smoothed.
	 */
	public void setMatrix(float[][][] m){ 
		matrix = m; 
		setMatrixSize(matrix.length);
	}
	
	/**
	 * getMatrix
	 * @return float[][][] the matrix that has been smoothed.
	 */
	public float[][][] getMatrix(){ return matrix; }
	
	/**
	 * setMatrixSize
	 * @param int size - set the size of the matrix.
	 */
	public void setMatrixSize(int size){ matrix_size = size; }
	
	/**
	 * getMatrixSize
	 * @return the matrix size.
	 */
	public int getMatrixSize(){ return matrix_size; }
	
	/**
	 * smooth
	 * abstract method, must be implemented in every subclass of Filter, this is the 
	 * method that should do the actual smoothing of the heightField.
	 */
	public abstract void smooth();
	
	/**
	 * getFilterName
	 * @return String - return the name of the filter.
	 */
	public String getFilterName(){ return filter_name; }
	
	/**
	 * setParameter
	 * @param float p - set the input_parameter that might be used by the subclasses
	 */
	public void setParameter(float p){ input_parameter = p; }
	
	/**
	 * getParameter
	 * @return float input_parameter;
	 */
	public float getParameter(){ return input_parameter; }
}