package terrain.filter;

/***********************************************************************************************
 * GAUSSIAN FILTER
 * date: 2004-05-18
 * author: Kristian Lunde
 * 
 * To make this gaussianfilter work correctly it has to be assumed that x, y and z in the are 
 * distributed by the normaldistribution N(0 SIGMA), and that they are stocastic variables.
************************************************************************************************/
 
public class GaussianFilter extends Filter{

	/* Euler's number*/
	private static float E = (float)Math.E;
	/* PI */
	private static float PI = (float)Math.PI;
	/* SIGMA used by the gauss method */
	private final static float SIGMA = 0.5f;
	/* DELTA used by the laplace method */
	private final static float DELTA = 1.0f;

	/**
	 * Constructor
	 * @param float[][][] matrix - the heightfield that should be manipulated.
	 */
	public GaussianFilter(float[][][] matrix){
		super(matrix);
		filter_name = "Gaussian filter";
	}
	
	/**
	 * smooth
	 * smooth the heightfield.
	 */
	public void smooth(){
		for(int x = 0; x < getMatrixSize(); x++){
			for(int y = 0; y < getMatrixSize(); y++){
				matrix[x][y][2] = laplace(gauss(matrix[x][y][0], matrix[x][y][1]), matrix[x][y][0], matrix[x][y][1]) * matrix[x][y][2];
			}
		}
	}
	
	/**
	 * laplace
	 * @param float gauss - is the calculated gaussian value based on the x and y coordinate of the heightfield.
	 * @param float x - a x coordinate from the heightfield.
	 * @param float y - a y coordinate from the heightfield.
	 * return float - the laplacian calculation of the input data.
	 */
	private float laplace(float gauss, float x, float y){
		return (((DELTA * DELTA) * gauss) / DELTA * (x * x)) +
					 (((DELTA * DELTA) * gauss) / DELTA * (y * y));
	}
	
	/**
	 * gauss
	 * @param float x - a x coordinate from the heightfield.
	 * @param float y - a y coordinate from the heightfield.
	 * @return float - the gaussian value of the x and y coordinates.
	 */
	private float gauss(float x, float y){
		return (1 / (2* PI * (SIGMA * SIGMA))) * 
					 (float)Math.exp(-((x * x) + (y * y) / (2.0f * (SIGMA * SIGMA))));
	}
}